#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include "ATmega88PA.h"
#include "device.h"
#include "stub.h"
#include "rtc.h"

static RTC_TIME time = {0, 0, 0};

RTC_TIME RTC_GetTime(void) {
    return time;
}

void RTC_ChangeMinute(int8_t offset) {
    time.second = 0;
    time.minute += offset + 60;
    time.minute %= 60;
}

void RTC_ChangeHour(int8_t offset) {
    time.second = 0;
    time.hour += offset + 24;
    time.hour %= 24;
}

void RTC_Init(void) {
    TIMER2_NORMAL_MODE;
    TIMER2_ASYNCHRONOUS_CLOCK_INPUT;
    TIMER2_CLEAR_OVERFLOW_INTERRUPT_FLAG;
    TIMER2_OVERFLOW_INTERRUPT_ENABLE;
    TIMER2_CLOCK_PRESCALER_128_START;
    RTC_Tick = STUB_void;
}

ISR(TIMER2_OVF_vect) {
    if(++time.second == 60) {
        time.second = 0;
        if(++time.minute == 60) {
            time.minute = 0;
            if(++time.hour == 24) {
                time.hour = 0;
            }
        }
    }
    if(POWER_ON) {
        RTC_Tick();
        DEVICE_Wakeup();
    } else {
        // Make sure that one TOSC cycle has elapsed
        TIMER2_CLOCK_PRESCALER_128_START;
        while(TIMER2_PRESCALER_ASYNC_UPDATE) {}
    }
    //reset watchdog
    wdt_reset();
}
